@@ -4,15 +4,13 @@ use crate::ui::{Cell, Row, Table};
 
 #[derive(Clone, Debug, Default, Serialize)]
 pub struct Folder {
-    pub delim: String,
     pub name: String,
     pub desc: String,
 }
 
-impl From<&pimalaya_email::Folder> for Folder {
-    fn from(folder: &pimalaya_email::Folder) -> Self {
+impl From<&pimalaya_email::folder::Folder> for Folder {
+    fn from(folder: &pimalaya_email::folder::Folder) -> Self {
         Folder {
-            delim: folder.delim.clone(),
             name: folder.name.clone(),
             desc: folder.desc.clone(),
         }
@@ -22,14 +20,12 @@ impl From<&pimalaya_email::Folder> for Folder {
 impl Table for Folder {
     fn head() -> Row {
         Row::new()
-            .cell(Cell::new("DELIM").bold().underline().white())
             .cell(Cell::new("NAME").bold().underline().white())
             .cell(Cell::new("DESC").bold().underline().white())
     }
 
     fn row(&self) -> Row {
         Row::new()
-            .cell(Cell::new(&self.delim).white())
             .cell(Cell::new(&self.name).blue())
             .cell(Cell::new(&self.desc).green())
     }
