@@ -1,7 +1,6 @@
-use std::ops;
-
 use anyhow::Result;
 use serde::Serialize;
+use std::ops;
 
 use crate::{
     printer::{PrintTable, PrintTableOpts, WriteColor},
@@ -20,8 +19,8 @@ impl ops::Deref for Folders {
     }
 }
 
-impl From<pimalaya_email::Folders> for Folders {
-    fn from(folders: pimalaya_email::Folders) -> Self {
+impl From<pimalaya_email::folder::Folders> for Folders {
+    fn from(folders: pimalaya_email::folder::Folders) -> Self {
         Folders(folders.iter().map(Folder::from).collect())
     }
 }
