@@ -1,9 +1,12 @@
 use anyhow::{anyhow, Context, Result};
 use clap::Command;
+use log::{debug, warn};
 #[cfg(feature = "imap-backend")]
-use pimalaya_email::ImapBackend;
+use pimalaya_email::backend::ImapBackend;
 use pimalaya_email::{
-    BackendBuilder, BackendConfig, BackendSyncBuilder, SenderBuilder, DEFAULT_INBOX_FOLDER,
+    account::{sync::AccountSyncBuilder, DEFAULT_INBOX_FOLDER},
+    backend::{BackendBuilder, BackendConfig},
+    sender::SenderBuilder,
 };
 use std::env;
 use url::Url;
@@ -45,6 +48,12 @@ fn create_app() -> Command {
 #[allow(clippy::single_match)]
 #[tokio::main]
 async fn main() -> Result<()> {
+    #[cfg(not(target_os = "windows"))]
+    if let Err((_, err)) = coredump::register_panic_handler() {
+        warn!("cannot register custom panic handler: {err}");
+        debug!("cannot register custom panic handler: {err:?}");
+    }
+
     let default_env_filter = env_logger::DEFAULT_FILTER_ENV;
     env_logger::init_from_env(env_logger::Env::default().filter_or(default_env_filter, "off"));
 
@@ -54,17 +63,20 @@ async fn main() -> Result<()> {
         let url = Url::parse(&raw_args[1])?;
         let config = DeserializedConfig::from_opt_path(None)?;
         let account_config = config.to_account_config(None)?;
-        let mut backend = BackendBuilder::new(account_config.clone()).build()?;
-        let mut sender = SenderBuilder::new(account_config.clone()).build()?;
+        let mut backend = BackendBuilder::new(account_config.clone()).build().await?;
+        let mut sender = SenderBuilder::new(account_config.clone()).build().await?;
         let mut printer = StdoutPrinter::default();
 
-        return email::handlers::mailto(
+        email::handlers::mailto(
             &account_config,
             backend.as_mut(),
             sender.as_mut(),
             &mut printer,
             &url,
-        );
+        )
+        .await?;
+
+        return Ok(());
     }
 
     let app = create_app();
@@ -89,7 +101,6 @@ async fn main() -> Result<()> {
 
     let config = DeserializedConfig::from_opt_path(config::args::parse_arg(&m))?;
     let account_config = config.to_account_config(account::args::parse_arg(&m))?;
-    let account_name = account_config.name.clone();
     let folder = folder::args::parse_source_arg(&m);
     let disable_cache = cache::args::parse_disable_cache_flag(&m);
 
@@ -103,17 +114,19 @@ async fn main() -> Result<()> {
 
     #[cfg(feature = "imap-backend")]
     if let BackendConfig::Imap(imap_config) = &account_config.backend {
-        let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
+        let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
         match imap::args::matches(&m)? {
             Some(imap::args::Cmd::Notify(keepalive)) => {
                 let mut backend =
-                    ImapBackend::new(account_config.clone(), imap_config.clone(), None)?;
-                return imap::handlers::notify(&mut backend, &folder, keepalive);
+                    ImapBackend::new(account_config.clone(), imap_config.clone(), None).await?;
+                imap::handlers::notify(&mut backend, &folder, keepalive).await?;
+                return Ok(());
             }
             Some(imap::args::Cmd::Watch(keepalive)) => {
                 let mut backend =
-                    ImapBackend::new(account_config.clone(), imap_config.clone(), None)?;
-                return imap::handlers::watch(&mut backend, &folder, keepalive);
+                    ImapBackend::new(account_config.clone(), imap_config.clone(), None).await?;
+                imap::handlers::watch(&mut backend, &folder, keepalive).await?;
+                return Ok(());
             }
             _ => (),
         }
@@ -121,17 +134,20 @@ async fn main() -> Result<()> {
 
     match account::args::matches(&m)? {
         Some(account::args::Cmd::List(max_width)) => {
-            return account::handlers::list(max_width, &account_config, &config, &mut printer);
+            account::handlers::list(max_width, &account_config, &config, &mut printer)?;
+            return Ok(());
         }
         Some(account::args::Cmd::Sync(strategy, dry_run)) => {
-            let sync_builder = BackendSyncBuilder::new(account_config, backend_builder)?
+            let sync_builder = AccountSyncBuilder::new(account_config, backend_builder)
+                .await?
                 .with_some_folders_strategy(strategy)
                 .with_dry_run(dry_run);
-            account::handlers::sync(&mut printer, sync_builder, dry_run)?;
+            account::handlers::sync(&mut printer, sync_builder, dry_run).await?;
             return Ok(());
         }
         Some(account::args::Cmd::Configure(reset)) => {
-            return account::handlers::configure(&account_config, reset);
+            account::handlers::configure(&account_config, reset).await?;
+            return Ok(());
         }
         _ => (),
     }
@@ -142,31 +158,27 @@ async fn main() -> Result<()> {
             let folder = folder
                 .ok_or_else(|| anyhow!("the folder argument is missing"))
                 .context("cannot create folder")?;
-            let folder = account_config.folder_alias(folder)?;
-            let mut backend = backend_builder.build()?;
-            return folder::handlers::create(&mut printer, backend.as_mut(), &folder);
+            let mut backend = backend_builder.build().await?;
+            folder::handlers::create(&mut printer, backend.as_mut(), &folder).await?;
+            return Ok(());
         }
         Some(folder::args::Cmd::List(max_width)) => {
-            let mut backend = backend_builder.build()?;
-            return folder::handlers::list(
-                &account_config,
-                &mut printer,
-                backend.as_mut(),
-                max_width,
-            );
+            let mut backend = backend_builder.build().await?;
+            folder::handlers::list(&account_config, &mut printer, backend.as_mut(), max_width)
+                .await?;
+            return Ok(());
         }
         Some(folder::args::Cmd::Expunge) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.build()?;
-            return folder::handlers::expunge(&mut printer, backend.as_mut(), &folder);
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.build().await?;
+            folder::handlers::expunge(&mut printer, backend.as_mut(), &folder).await?;
+            return Ok(());
         }
         Some(folder::args::Cmd::Delete) => {
-            let folder = folder
-                .ok_or_else(|| anyhow!("the folder argument is missing"))
-                .context("cannot delete folder")?;
-            let folder = account_config.folder_alias(folder)?;
-            let mut backend = backend_builder.build()?;
-            return folder::handlers::delete(&mut printer, backend.as_mut(), &folder);
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.build().await?;
+            folder::handlers::delete(&mut printer, backend.as_mut(), &folder).await?;
+            return Ok(());
         }
         _ => (),
     }
@@ -174,54 +186,54 @@ async fn main() -> Result<()> {
     // checks email commands
     match email::args::matches(&m)? {
         Some(email::args::Cmd::Attachments(ids)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
-            return email::handlers::attachments(
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
+            email::handlers::attachments(
                 &account_config,
                 &mut printer,
                 &id_mapper,
                 backend.as_mut(),
                 &folder,
                 ids,
-            );
+            )
+            .await?;
+            return Ok(());
         }
         Some(email::args::Cmd::Copy(ids, to_folder)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::copy(
-                &account_config,
+            email::handlers::copy(
                 &mut printer,
                 &id_mapper,
                 backend.as_mut(),
                 &folder,
                 to_folder,
                 ids,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Delete(ids)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::delete(
-                &account_config,
-                &mut printer,
-                &id_mapper,
-                backend.as_mut(),
-                &folder,
-                ids,
-            );
+            email::handlers::delete(&mut printer, &id_mapper, backend.as_mut(), &folder, ids)
+                .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Forward(id, headers, body)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let mut sender = sender_builder.build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let mut sender = sender_builder.build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::forward(
+            email::handlers::forward(
                 &account_config,
                 &mut printer,
                 &id_mapper,
@@ -231,14 +243,17 @@ async fn main() -> Result<()> {
                 id,
                 headers,
                 body,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::List(max_width, page_size, page)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::list(
+            email::handlers::list(
                 &account_config,
                 &mut printer,
                 &id_mapper,
@@ -247,29 +262,34 @@ async fn main() -> Result<()> {
                 max_width,
                 page_size,
                 page,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Move(ids, to_folder)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::move_(
-                &account_config,
+            email::handlers::move_(
                 &mut printer,
                 &id_mapper,
                 backend.as_mut(),
                 &folder,
                 to_folder,
                 ids,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Read(ids, text_mime, raw, headers)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::read(
+            email::handlers::read(
                 &account_config,
                 &mut printer,
                 &id_mapper,
@@ -279,15 +299,18 @@ async fn main() -> Result<()> {
                 text_mime,
                 raw,
                 headers,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Reply(id, all, headers, body)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let mut sender = sender_builder.build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let mut sender = sender_builder.build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::reply(
+            email::handlers::reply(
                 &account_config,
                 &mut printer,
                 &id_mapper,
@@ -298,28 +321,33 @@ async fn main() -> Result<()> {
                 all,
                 headers,
                 body,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Save(raw_email)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::save(
-                &account_config,
+            email::handlers::save(
                 &mut printer,
                 &id_mapper,
                 backend.as_mut(),
                 &folder,
                 raw_email,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Search(query, max_width, page_size, page)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::search(
+            email::handlers::search(
                 &account_config,
                 &mut printer,
                 &id_mapper,
@@ -329,14 +357,17 @@ async fn main() -> Result<()> {
                 max_width,
                 page_size,
                 page,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Sort(criteria, query, max_width, page_size, page)) => {
-            let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-            let mut backend = backend_builder.clone().into_build()?;
-            let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+            let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+            let mut backend = backend_builder.clone().into_build().await?;
+            let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-            return email::handlers::sort(
+            email::handlers::sort(
                 &account_config,
                 &mut printer,
                 &id_mapper,
@@ -347,71 +378,86 @@ async fn main() -> Result<()> {
                 max_width,
                 page_size,
                 page,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Send(raw_email)) => {
-            let mut backend = backend_builder.build()?;
-            let mut sender = sender_builder.build()?;
-            return email::handlers::send(
+            let mut backend = backend_builder.build().await?;
+            let mut sender = sender_builder.build().await?;
+            email::handlers::send(
                 &account_config,
                 &mut printer,
                 backend.as_mut(),
                 sender.as_mut(),
                 raw_email,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         Some(email::args::Cmd::Flag(m)) => match m {
             Some(flag::args::Cmd::Set(ids, ref flags)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-                return flag::handlers::set(
+                flag::handlers::set(
                     &mut printer,
                     &id_mapper,
                     backend.as_mut(),
                     &folder,
                     ids,
                     flags,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             Some(flag::args::Cmd::Add(ids, ref flags)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-                return flag::handlers::add(
+                flag::handlers::add(
                     &mut printer,
                     &id_mapper,
                     backend.as_mut(),
                     &folder,
                     ids,
                     flags,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             Some(flag::args::Cmd::Remove(ids, ref flags)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-                return flag::handlers::remove(
+                flag::handlers::remove(
                     &mut printer,
                     &id_mapper,
                     backend.as_mut(),
                     &folder,
                     ids,
                     flags,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             _ => (),
         },
         Some(email::args::Cmd::Tpl(m)) => match m {
             Some(tpl::args::Cmd::Forward(id, headers, body)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-                return tpl::handlers::forward(
+                tpl::handlers::forward(
                     &account_config,
                     &mut printer,
                     &id_mapper,
@@ -420,17 +466,21 @@ async fn main() -> Result<()> {
                     id,
                     headers,
                     body,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             Some(tpl::args::Cmd::Write(headers, body)) => {
-                return tpl::handlers::write(&account_config, &mut printer, headers, body);
+                tpl::handlers::write(&account_config, &mut printer, headers, body).await?;
+                return Ok(());
             }
             Some(tpl::args::Cmd::Reply(id, all, headers, body)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-                return tpl::handlers::reply(
+                tpl::handlers::reply(
                     &account_config,
                     &mut printer,
                     &id_mapper,
@@ -440,48 +490,60 @@ async fn main() -> Result<()> {
                     all,
                     headers,
                     body,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             Some(tpl::args::Cmd::Save(tpl)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let id_mapper = IdMapper::new(backend.as_ref(), &account_name, &folder)?;
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let id_mapper = IdMapper::new(backend.as_ref(), &account_config, &folder)?;
 
-                return tpl::handlers::save(
+                tpl::handlers::save(
                     &account_config,
                     &mut printer,
                     &id_mapper,
                     backend.as_mut(),
                     &folder,
                     tpl,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             Some(tpl::args::Cmd::Send(tpl)) => {
-                let folder = account_config.folder_alias(folder.unwrap_or(DEFAULT_INBOX_FOLDER))?;
-                let mut backend = backend_builder.clone().into_build()?;
-                let mut sender = sender_builder.build()?;
-                return tpl::handlers::send(
+                let folder = folder.unwrap_or(DEFAULT_INBOX_FOLDER);
+                let mut backend = backend_builder.clone().into_build().await?;
+                let mut sender = sender_builder.build().await?;
+                tpl::handlers::send(
                     &account_config,
                     &mut printer,
                     backend.as_mut(),
                     sender.as_mut(),
                     &folder,
                     tpl,
-                );
+                )
+                .await?;
+
+                return Ok(());
             }
             _ => (),
         },
         Some(email::args::Cmd::Write(headers, body)) => {
-            let mut backend = backend_builder.build()?;
-            let mut sender = sender_builder.build()?;
-            return email::handlers::write(
+            let mut backend = backend_builder.build().await?;
+            let mut sender = sender_builder.build().await?;
+            email::handlers::write(
                 &account_config,
                 &mut printer,
                 backend.as_mut(),
                 sender.as_mut(),
                 headers,
                 body,
-            );
+            )
+            .await?;
+
+            return Ok(());
         }
         _ => (),
     }
